package MT::Plugin::PostTwiOauth;

use strict;
use base qw( MT::Plugin );
use CGI;
use MT;
use MT::Blog;
use MT::Plugin;
use MT::Util qw(encode_url);
use HTTP::Request::Common;
use LWP::UserAgent;
use Digest::SHA1;
use Net::OAuth;
$Net::OAuth::PROTOCOL_VERSION = Net::OAuth::PROTOCOL_VERSION_1_0A; 

our $PLUGIN_NAME = 'PostTwiOauth';
our $VERSION = '0.50';
our $SCHEMA_VERSION = '0.32';

my $plugin = new MT::Plugin::PostTwiOauth({
    name => $PLUGIN_NAME,
    key => $PLUGIN_NAME,
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
    description => "Post entry title and bit.ly permalink to Twitter with OAuth",
    author_name => 'macminiosx',
    author_link => 'http://www.macminiosx.com/',
    doc_link => 'http://www.macminiosx.com/tag/PostTwiOauth',
    blog_config_template => \&_posttwioauth_config,
	settings => new MT::PluginSettings([
		['consumer_key'],
		['consumer_secret'],
		['callback_url'],
		['access_token'],
		['access_secret'],
		['bitly_user'],
		['bitly_api'],
		['hashtag'],
		['tweet_title'],
		['rt_twitter_name'],
		['rt_hashtag'],
		['rt_title'],
		['utm_source'],
		['utm_medium'],
		['utm_campaign'],
	]),
	object_classes => [
		'MT::PostTwiOauth',
	],

});

MT->add_plugin($plugin);

sub instance { $plugin; }

sub init_registry {
	my $plugin = shift;
	$plugin->registry({
		callbacks => {
			'cms_post_save.entry' => \&_pto_ent,
			'cms_post_save.page' => \&_pto_ent,
			'api_post_save.entry' => \&_pto_ent,
			'api_post_save.page' => \&_pto_ent,
			'scheduled_post_published' => \&_pto_ent,
			'MT::App::CMS::template_source.edit_entry' => \&_edit_tmpl,
		},
		applications => {
			cms => {
				methods => {
					'posttwioauth_repost'  => \&_repost_pto_ent,
					'posttwioauth_setinfo' => \&_twi_setinfo,
					'posttwioauth_test'    => \&_test_post,
					'posttwioauth_req'     => \&_oauth_req,
					'posttwioauth_cb'      => \&_oauth_cb,
				},
			},
		},
		object_types => {
			'posttwioauth' => 'MT::PostTwiOauth',
		}
	});
}

sub _save_log {
	my ($error_mess,$blog_id,$author_id) = @_;
	use MT::Log; 
	my $log = MT::Log->new;
	$error_mess = $PLUGIN_NAME . ':' . $error_mess;
	$log->blog_id($blog_id);
	$log->author_id($author_id) if defined $author_id;
	$log->message($error_mess);
	$log->level(MT::Log::ERROR);
	$log->save
		or die $log->errstr;
	return;
}

sub _posttwioauth_config {
	my $plugin = shift;
	my ($param,  $blog_id) = @_;
	$blog_id =~ s!blog:!!;
	my $tmpl;
	
	$tmpl = $plugin->load_tmpl('posttwioauth_config.tmpl');
	$tmpl->param('blog_id' => $blog_id);
	$tmpl->param('magic_token' => MT->app->current_magic);
	$tmpl->param('mt_app_url' => MT->app->base . MT->app->mt_path);
	return $tmpl;
}

sub _posttwioauth_twitter_config {
	my ($app, $tmpl, $widget_param) = @_;
	my $blog_id = $app->{'query'}->param('blog_id');
	my %plugin_params;
	$plugin->load_config(\%plugin_params, 'blog:' . $blog_id);
	my $client_enabel = $plugin_params{'client_enabel'};
	$tmpl->param('client_enabel' => $client_enabel);
	my $screen_name = $plugin_params{'rt_twitter_name'};
	$tmpl->param('screen_name' => $screen_name);
	$tmpl->param('blog_id' => $blog_id);
	return 1;
}

sub save_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
	$param->{'access_token'} = $plugin->get_config_value('access_token', $scope);
	$param->{'access_secret'} = $plugin->get_config_value('access_secret', $scope);
	$param->{'rt_twitter_name'} = $plugin->get_config_value('rt_twitter_name', $scope);
	$plugin->SUPER::save_config($param, $scope);
}

sub _pto_ent {
	my ($eh, $app, $obj, $original, $rp, $rp_txt) = @_;
	return unless ref $obj eq 'MT::Entry';
	my $blog_id = $obj->blog_id;
	my $author_id;
	eval {
		$author_id = $app->user->id;
	};
	require MT::Entry;
	return unless $obj->status == MT::Entry::RELEASE();
	return if !$rp && $original && $original->status == MT::Entry::RELEASE();
	
	my %plugin_params;
	$plugin->load_config(\%plugin_params, 'blog:' . $blog_id);
	my $consumer_key = $plugin_params{'consumer_key'};
	return unless $consumer_key;
	my $consumer_secret = $plugin_params{'consumer_secret'};
	return _save_log('No Consumer secret',$blog_id, $author_id) unless $consumer_secret;
	my $access_token = $plugin_params{'access_token'};
	return _save_log('No Access token',$blog_id, $author_id) unless $access_token;
	my $access_secret = $plugin_params{'access_secret'};
	return _save_log('No Access secret',$blog_id, $author_id) unless $access_secret;
	my $bitly_user = $plugin_params{'bitly_user'};
	return _save_log('No bit.ly user name',$blog_id, $author_id) unless $bitly_user;
	my $bitly_api = $plugin_params{'bitly_api'};
	return _save_log('No bit.ly API key',$blog_id, $author_id) unless $bitly_api;
	my $hashtag = $plugin_params{'hashtag'};
	if($hashtag) {
		$hashtag = '#' . $hashtag if $hashtag !~ /^\#/;
		$hashtag = ' ' . $hashtag;
	}
	my $tweet_title = $plugin_params{'tweet_title'};
	my $rt_twitter_name = $plugin_params{'rt_twitter_name'};
	my $utm_source = $plugin_params{'utm_source'};
	my $utm_medium = $plugin_params{'utm_medium'};
	my $utm_campaign = $plugin_params{'utm_campaign'};
	my %campaign_code;
	if ($utm_source && $utm_medium && $utm_campaign) {
		%campaign_code = (
			'utm_source'   => $utm_source,
			'utm_medium'   => $utm_medium,
			'utm_campaign' => $utm_campaign
		);
	}
	
	my $max_tweet_length = 140;
	my $cut_str_length = 3;
	my $cut_str = '...';
	my $etc_length = 1;
	
	require MT::I18N;
	my $title = $obj->title;
    my $title_length = MT::I18N::length_text($title);
    
    my $entry_url = $obj->permalink;
    if (%campaign_code) {
		require URI;
		my $uri = URI->new($entry_url);
		%campaign_code = (%campaign_code , $uri->query_form);
		$uri->query_form( \%campaign_code );
		$entry_url = $uri->as_string;
	}
    my $url = makeashorterlink($entry_url, $bitly_user, $bitly_api, $blog_id, $author_id);
	my $url_length = $url ? MT::I18N::length_text($url) : 0;
	
	my $hashtag_length = $hashtag ? MT::I18N::length_text($hashtag) : 0;
	$tweet_title = $tweet_title ? $tweet_title . ' ' : '';
	my $tweet_title_length = $tweet_title ? MT::I18N::length_text($tweet_title) : 0;
	$rp_txt = $rp_txt ? $rp_txt . ' ' : '';
	my $rp_txt_length = $rp_txt ? MT::I18N::length_text($rp_txt) : 0;
	my $comment_length = $title_length + $url_length + $hashtag_length + $tweet_title_length + $rp_txt_length + $etc_length;
	my $subtraction_length = $url_length + $hashtag_length + $tweet_title_length + $rp_txt_length + $etc_length + $cut_str_length;
	
	if($comment_length > $max_tweet_length) {
		$title_length = ($max_tweet_length - $subtraction_length);
		$title = MT::I18N::substr_text($title, 0, $title_length) . $cut_str;
	}
	
	my $message = $rp_txt . $tweet_title . "$title $url" . $hashtag;
	my $charset = $app->charset;
	$message = MT::I18N::encode_text($message, $charset, 'utf-8') if $charset ne 'UTF-8';
	
	my $API_Method_url  = 'https://api.twitter.com/1.1/statuses/update.json';
	my $request_method = 'POST'; 
	
	my $request  = Net::OAuth->request("protected resource")->new(
		consumer_key    => $consumer_key,
		consumer_secret => $consumer_secret,
		request_url => $API_Method_url,
		request_method => $request_method,
		signature_method => 'HMAC-SHA1',
		timestamp => time,
		nonce => Digest::SHA1::sha1_base64(time . $$ . rand),
		token => $access_token,
		token_secret => $access_secret,
		extra_params => { status => $message },
	);

	$request->sign;
	return _save_log('Signature verification failed! check OAuth parameters.',$blog_id, $author_id)
		unless $request->verify;

	my $ua = LWP::UserAgent->new(
		ssl_opts => {
			verify_hostname => 1
		},
	);
	my $http_hdr = HTTP::Headers->new('User-Agent' => $PLUGIN_NAME);
	my $http_req = HTTP::Request->new($request_method, $API_Method_url, $http_hdr, $request->to_post_body);
	$http_req->content_type('application/x-www-form-urlencoded');
	my $res = $ua->request($http_req);
	my $error_message = $res->as_string unless $res->is_success;
	return _save_log("Failed to post to twitter.$error_message",$blog_id, $author_id) unless $res->is_success;
	require JSON;
	require MT::PostTwiOauth;
	my $tobj = JSON::from_json($res->content);
	my $status_id = $tobj->{'id'};
	my $twi = MT::PostTwiOauth->load({id => $obj->id});
	if (defined $twi) {
		$twi->twi_name($rt_twitter_name);
		$twi->twi_sid($status_id);
		$twi->bitly_url($url);
		$twi->modified_on($obj->modified_on);
		$twi->save;
	}else {
		$twi = MT::PostTwiOauth->new;
		$twi->id($obj->id);
		$twi->blog_id($blog_id);
		$twi->twi_name($rt_twitter_name);
		$twi->twi_sid($status_id);
		$twi->bitly_url($url);
		$twi->modified_on($obj->modified_on);
		$twi->save;
	}
	return 1;
}
sub makeashorterlink {
	my ($entry_link, $bitly_user, $bitly_api, $blog_id, $author_id) = @_;
	my $agent = $PLUGIN_NAME;
	my $ua = LWP::UserAgent->new(agent => $agent);
	my $biturl = "http://api.bit.ly/v3/shorten";
	my $res = $ua->post($biturl, [
		'longUrl' => $entry_link,
		'login' => $bitly_user,
		'apiKey' => $bitly_api,
		'format' => 'json',
	]);
	$res->is_success or return _save_log('Failed to get response from bit.ly',$blog_id, $author_id);
	require JSON;
	my $obj = JSON::from_json($res->content) or return _save_log('Failed to get  shortened url from bit.ly',$blog_id, $author_id);
	return _save_log('Failed to get  shortened url from bit.ly',$blog_id, $author_id) if $obj->{errorCode};
	my $bitlyurl = $obj->{data}->{url};
	return $bitlyurl;
}
sub _edit_tmpl {
	my ($cb, $app, $tmpl) = @_;
	my $entry_id = $app->{'query'}->param('id');
	my $blog_id = $app->{'query'}->param('blog_id');
	my $access_token = $plugin->get_config_value( 'access_token', 'blog:' . $blog_id);
	return unless $access_token;
	require MT::PostTwiOauth;
	my $twi = MT::PostTwiOauth->load({id => $entry_id});
	my ($twi_name,$twi_sid,$bitly_url,$tweet_info,$info_label);
	if (defined $twi) {
		$twi_name = $twi->twi_name if $twi->twi_name;
		$twi_sid = $twi->twi_sid if $twi->twi_sid;
		$bitly_url = $twi->bitly_url if $twi->bitly_url;
	}
	if(defined $twi_name && defined $twi_sid && defined $bitly_url) {
		$info_label = 'Posted Twitter info';
		my $tweet_url = 'http://twitter.com/' . $twi_name . '/status/' . $twi_sid;
		my $bit_info = $bitly_url . '+';
		$tweet_info =<<"EOF";
<dl>
<dt>Twitter link:</dt>
<dd><a href="$tweet_url" target="_blank">$tweet_url</a></dd>
<dt>Shortened URL:</dt>
<dd><a href="$bitly_url" target="_blank">$bitly_url</a></dd>
EOF
		if($bitly_url =~ /bit.ly/) {
			$tweet_info .=<<"EOF";
<dt>bit.ly info page:</dt>
<dd><a href="$bit_info" target="_blank">$bit_info</a></dd>
</dl>
EOF
		}else {
			$tweet_info .=<<"EOF";
</dl>
EOF
		}
	}else {
		$info_label = 'Set Twitter info';
		$tweet_info =<<"EOF";
<dl>
<dt>Twitter Status ID</dt>
<dd><input type="text" name="twi_sid" size="20" value="$twi_sid" /><br />
ex: 1234567890</dd>
<dt>Shortened URL:</dt>
<dd><input type="text" name="bitly_url" size="20" value="$bitly_url" />&nbsp;(include http://)<br />
ex: http://bit.ly/xxxxxx</dd>
</dl>
<button mt:mode="posttwioauth_setinfo" name="setinfo_button"
type="submit"
title="Set"
class="action"
>Set</button>
EOF
	}
	
	my $old = <<"END_TMPL";
</mt:setvarblock>

<mt:setvarblock name="js_include" append="1">
END_TMPL
	my $new = <<"END_TMPL";
<mt:unless name="new_object">
<mtapp:widget
	id="posttwioauth-widget"
	label="PostTwiOAuth Repost">
	<div class="actions-bar">
	Add Repost text(option):<br />
	If you need "Repost text"<br />
	<input type="text" name="rp_txt" size="20" /><br />
	<button mt:mode="posttwioauth_repost" name="repost_button"
	type="submit"
	title="Repost"
	class="action"
	>Repost</button>
	</div>
</mtapp:widget>
<mtapp:widget
	id="posttwioauth-twiinfo"
	label="$info_label">
	<div class="actions-bar">
	$tweet_info
	</div>
</mtapp:widget>
</mt:unless>
END_TMPL
	$$tmpl =~ s{($old)}{$new$1}m;
	
	1;
}
sub _repost_pto_ent {
	my $app = shift;
	my $type = $app->{'query'}->param('_type');
	my $obj_id = $app->{'query'}->param('id');
	my $blog_id = $app->{'query'}->param('blog_id');
	my $rp_txt = $app->{'query'}->param('rp_txt');
	my $class = $app->model($type) || 'entry';
	my $obj = $class->load($obj_id);
	my $original = $obj->clone;
	my $rp = 1;
	my $res = _pto_ent(undef, $app, $obj, $original, $rp, $rp_txt);
	return $app->redirect(
		$app->uri(
			mode => 'view',
			args => {
				'_type' => $type,
				id => $obj_id,
				blog_id => $blog_id,
			}
		)
	);
}
sub _twi_setinfo {
	my $app = shift;
	my $type = $app->{'query'}->param('_type');
	my $obj_id = $app->{'query'}->param('id');
	my $blog_id = $app->{'query'}->param('blog_id');
	my $twi_sid = $app->{'query'}->param('twi_sid');
	my $bitly_url = $app->{'query'}->param('bitly_url');
	my $class = $app->model($type) || 'entry';
	my $obj = $class->load($obj_id);
	if(defined $twi_sid && $twi_sid !~ /\D/ && defined $bitly_url && $bitly_url =~ /^http:\/\//) {
		require MT::PostTwiOauth;
		my $twi = MT::PostTwiOauth->load({id => $obj_id});
		if (defined $twi) {
			my $twi_name = $twi->twi_name;
			unless ($twi_name) {
				my %plugin_params;
				$plugin->load_config(\%plugin_params, 'blog:' . $blog_id);
				$twi_name = $plugin_params{'rt_twitter_name'};
			}
			$twi->twi_name($twi_name);
			$twi->twi_sid($twi_sid);
			$twi->bitly_url($bitly_url);
			$twi->modified_on($obj->modified_on);
			$twi->save;
		}else {
			my %plugin_params;
			$plugin->load_config(\%plugin_params, 'blog:' . $blog_id);
			my $twi_name = $plugin_params{'rt_twitter_name'};
			$twi = MT::PostTwiOauth->new;
			$twi->id($obj_id);
			$twi->blog_id($blog_id);
			$twi->twi_name($twi_name);
			$twi->twi_sid($twi_sid);
			$twi->bitly_url($bitly_url);
			$twi->modified_on($obj->modified_on);
			$twi->save;
		}
		
	}
	return $app->redirect(
		$app->uri(
			mode => 'view',
			args => {
				'_type' => $type,
				id => $obj_id,
				blog_id => $blog_id,
			}
		)
	);
}
sub _oauth_req {
	my $app = shift;
	require PostTwiOauth::PostTwiOauthReq;
	PostTwiOauth::PostTwiOauthReq::OauthReq($app);
}
sub _oauth_cb {
	my $app = shift;
	require PostTwiOauth::PostTwiOauthCb;
	my %param = PostTwiOauth::PostTwiOauthCb::OauthCb($app);
	$app->{ plugin_template_path } = File::Spec->catdir( $plugin->path,'tmpl' );
	my $tmpl = 'posttwioauth_dance.tmpl';
	return $app->build_page( $tmpl, \%param );
}
sub _test_post {
	my $app = shift;
	require PostTwiOauth::PostTest;
	my %param = PostTwiOauth::PostTest::PostTestOauth($app);
	$app->{ plugin_template_path } = File::Spec->catdir( $plugin->path,'tmpl' );
	my $tmpl = 'posttwioauth_dance.tmpl';
	return $app->build_page( $tmpl, \%param );
}

1;
