#name => PostTwiOauth
#version => 0.50,
#description => 'Post entry title and bit.ly permalink to Twitter with OAuth',
#author_name => 'macminiosx',
#author_link => 'http://www.macminiosx.com/',
#doc_link => 'http://www.macminiosx.com/tag/PostTwiOauth',

package PostTwiOauth::PostTest;

use strict;
use CGI;
use MT;
use MT::App;
use MT::Plugin;
use HTTP::Request::Common;
use LWP::UserAgent;
use Digest::SHA1;
use Net::OAuth;
$Net::OAuth::PROTOCOL_VERSION = Net::OAuth::PROTOCOL_VERSION_1_0A;

our $PLUGIN_NAME = 'PostTwiOauth';
our $VERSION = '0.50';

sub _save_log {
	my ($error_mess,$blog_id,$author_id,$remote_ip) = @_;
	use MT::Log; 
	my $log = MT::Log->new;
	$error_mess = $PLUGIN_NAME . ':' . $error_mess;
	$log->blog_id($blog_id);
	$log->author_id($author_id);
	$log->ip($remote_ip) if $remote_ip;
	$log->message($error_mess);
	$log->level(MT::Log::ERROR);
	$log->save
		or die $log->errstr;
	return;
}
sub PostTestOauth {
	my $app = shift;
	my $blog_id = $app->{'query'}->param('blog_id');
	my $author_id = $app->user->id;
	my $remote_ip = $app->remote_ip;
	return _save_log('Illegal access!',$blog_id, $author_id, $remote_ip) unless $app->validate_magic;
	my $plugin = new MT::Plugin::PostTwiOauth({
    	name => $PLUGIN_NAME,
	});
	my %plugin_params;
	$plugin->load_config(\%plugin_params, 'blog:' . $blog_id);
	my $consumer_key = $plugin_params{'consumer_key'};
	return _save_log('No Consumer key',$blog_id, $author_id) unless $consumer_key;
	my $consumer_secret = $plugin_params{'consumer_secret'};
	return _save_log('No Consumer secret',$blog_id, $author_id) unless $consumer_secret;
	my $access_token = $plugin_params{'access_token'};
	return _save_log('No Access token',$blog_id, $author_id) unless $access_token;
	my $access_secret = $plugin_params{'access_secret'};
	return _save_log('No Access secret',$blog_id, $author_id) unless $access_secret;
	
	my $bitly_user = $plugin_params{'bitly_user'};
	my $bitly_api = $plugin_params{'bitly_api'};
	my($hashtag,$tweet_title,$blog,$url);
	require MT::Blog;
	if($bitly_user && $bitly_api) {
		$hashtag = $plugin_params{'hashtag'};
		if($hashtag) {
			$hashtag = '#' . $hashtag if $hashtag !~ /^\#/;
			$hashtag = ' ' . $hashtag;
		}
		$tweet_title = $plugin_params{'tweet_title'};
		$tweet_title .= ' ' if $tweet_title;
		
	    $blog = MT::Blog->load($blog_id);
		$url = ' ' . makeashorterlink($blog->site_url, $bitly_user, $bitly_api,$blog_id, $author_id);
	}
	require Encode;
	my $test_mess = Encode::decode( 'utf8', "PostTwiOauth $VERSION:Test Tweet ");
	Encode::_utf8_off($tweet_title);
	my $charset = $app->charset;
	$tweet_title = Encode::decode( $charset, $tweet_title);
	my $product_code = MT->product_code;
#	my $packs = MT->find_addons( 'pack' );
#	$product_code .= exists(@$packs->[0]->{label}) ? ' Pro ' : 'OS ';
	$test_mess .= $charset . ' ' . $product_code . mt->version_id . ' ';
	my $message = $tweet_title . $test_mess . time . $url . $hashtag;
    
	my $API_Method_url  = 'https://api.twitter.com/1.1/statuses/update.json';
	my $request_method = 'POST'; 

	my $request  = Net::OAuth->request("protected resource")->new(
		consumer_key    => $consumer_key,
		consumer_secret => $consumer_secret,
		request_url => $API_Method_url,
		request_method => $request_method,
		signature_method => 'HMAC-SHA1',
		timestamp => time,
		nonce => Digest::SHA1::sha1_base64(time . $$ . rand),
		token => $access_token,
		token_secret => $access_secret,
		extra_params => { status => $message },
	);

	$request->sign;
	return _save_log('Signature verification failed! check OAuth parameters.',$blog_id, $author_id)
		unless $request->verify;

	my $ua = LWP::UserAgent->new(
		ssl_opts => {
			verify_hostname => 1
		},
	);
	my $http_hdr = HTTP::Headers->new('User-Agent' => $PLUGIN_NAME);
	my $http_req = HTTP::Request->new($request_method, $API_Method_url, $http_hdr, $request->to_post_body);
	$http_req->content_type('application/x-www-form-urlencoded');
	my $res = $ua->request($http_req);
	my %param;
	if ($res->is_success) {
		$param{ 'page_title' } = 'Post to Twitter Success!';
		$param{ 'is_success' } = 1;
		$param{ 'show_twitter' } = 1;
		require JSON;
		my $obj = JSON::from_json($res->content);
		my $screen_name = $obj->{'user'}->{'screen_name'};
		my $status_id = $obj->{'id'};
		my $realName = $obj->{'user'}->{'name'};
		my $tweetText = $obj->{'text'};
		my $source = $obj->{'source'};
		my $profilePic = $obj->{'user'}->{'profile_image_url'};
		my $profileBackgroundColor = $obj->{'user'}->{'profile_background_color'};
		my $profileBackgroundImage = $obj->{'user'}->{'profile_background_image_url'};
		my $profileTextColor = $obj->{'user'}->{'profile_text_color'};
		my $profileLinkColor = $obj->{'user'}->{'profile_link_color'};
		my $tweetURL = "http://twitter.com/$screen_name/status/$status_id";
		$param{ 'msg' } = "Please check your twitter. [<a href='$tweetURL' target='_blank'>Go to this status.</a>]";

		$param{ 'Post_test' } =<<"EOF";
<style type='text/css'>.posttwioauth {background:url($profileBackgroundImage) #$profileBackgroundColor;padding:20px;} p.bbpTweet{background:#fff;padding:10px 12px 10px 12px;margin:0;min-height:48px;color:#000;font-size:18px !important;line-height:22px;-moz-border-radius:5px;-webkit-border-radius:5px} p.bbpTweet span.metadata{display:block;width:100%;clear:both;margin-top:8px;padding-top:12px;height:40px;border-top:1px solid #fff;border-top:1px solid #e6e6e6} p.bbpTweet span.metadata span.author{line-height:19px} p.bbpTweet span.metadata span.author img{float:left;margin:0 7px 0 0px;width:38px;height:38px} p.bbpTweet a:hover{text-decoration:underline}p.bbpTweet span.timestamp{font-size:12px;display:block}</style>
<div class='posttwioauth'><p class='bbpTweet'>$tweetText<span class='timestamp'><a href='$tweetURL' target='_blank'>less than a minute ago</a> via $source</span><span class='metadata'><span class='author'><a href='http://twitter.com/$screen_name' target='_blank'><img src='$profilePic' /></a><strong><a href='http://twitter.com/$screen_name' target='_blank'>$realName</a></strong><br/>$screen_name</span></span></p></div>
EOF
		if($charset ne 'UTF-8') {
			Encode::_utf8_off($param{ 'Post_test' });
			Encode::from_to($param{ 'Post_test' }, "utf8", $charset );
		}
		if($charset eq 'UTF-8' && MT->version_number < 5) {
			require MT::I18N;
			$param{ 'Post_test' } = MT::I18N::encode_text($param{ 'Post_test' }, undef ,'utf-8')
		}
	}else {
		$param{ 'page_title' } = 'Post to Twitter failure!';
		my $error_message = $res->as_string;
		$param{ 'msg' } = 'Test post failed. Please check your settings.<br />' . $error_message;
		_save_log("Test post failed.$error_message",$blog_id, $author_id);
	}
	return %param;

}
sub makeashorterlink {
	my ($blog_url, $bitly_user, $bitly_api,$blog_id,$author_id) = @_;
	my $agent = $PLUGIN_NAME;
	my $ua = LWP::UserAgent->new(agent => $agent);
	my $biturl = "http://api.bit.ly/v3/shorten";
	my $res = $ua->post($biturl, [
		'longUrl' => $blog_url,
		'login' => $bitly_user,
		'apiKey' => $bitly_api,
		'format' => 'json',
	]);
	$res->is_success or return _save_log('Failed to get response from bit.ly',$blog_id, $author_id);
	require JSON;
	my $obj = JSON::from_json($res->content) or return _save_log('Failed to get  shortened url from bit.ly',$blog_id, $author_id);
	return _save_log('Failed to get  shortened url from bit.ly',$blog_id, $author_id) if $obj->{errorCode};
	my $bitlyurl = $obj->{data}->{url};
	return $bitlyurl;
}

1;