#name => PostTwiOauth
#version => 0.50,
#description => 'Post entry title and bit.ly permalink to Twitter with OAuth',
#author_name => 'macminiosx',
#author_link => 'http://www.macminiosx.com/',
#doc_link => 'http://www.macminiosx.com/tag/PostTwiOauth',

package PostTwiOauth::PostTwiOauthCb;

use strict;
use CGI;
use MT;
use MT::App;
use MT::Plugin;
use HTTP::Request::Common;
use LWP::UserAgent;
use Digest::SHA1;
use Net::OAuth;
$Net::OAuth::PROTOCOL_VERSION = Net::OAuth::PROTOCOL_VERSION_1_0A;

our $PLUGIN_NAME = 'PostTwiOauth';

sub _save_log {
	my ($error_mess,$blog_id) = @_;
	use MT::Log; 
	my $log = MT::Log->new;
	$error_mess = $PLUGIN_NAME . ':' . $error_mess;
	$log->blog_id($blog_id);
	$log->message($error_mess);
	$log->level(MT::Log::ERROR);
	$log->save
		or die $log->errstr;
	return;
}
sub OauthCb {
	my $app = shift;
	my $c = CGI->new;
	my %cookies = $c->cookie('post2twi');
	my $blog_id = $cookies{'setBlog_id'};
	my $request_token = $cookies{'token'};
	my $request_token_secret = $cookies{'token_secret'};
	my $oauth_token = $app->param('oauth_token');
	my $verifier = $app->param('oauth_verifier');
	
	my $plugin = new MT::Plugin::PostTwiOauth({
    	name => $PLUGIN_NAME,
	});
	my %plugin_params;
	my $scope = 'blog:' . $blog_id;
	$plugin->load_config(\%plugin_params, $scope);
	my $consumer_key = $plugin_params{'consumer_key'};
	return q() unless $consumer_key;
	my $consumer_secret = $plugin_params{'consumer_secret'};
	return q() unless $consumer_secret;
    
	my $access_token_url = 'https://api.twitter.com/oauth/access_token';
	my $request_method = 'POST';

	my $request = Net::OAuth->request("access token")->new(
		consumer_key => $consumer_key,
		consumer_secret => $consumer_secret,
		request_url => $access_token_url,
		request_method => $request_method,
		signature_method => 'HMAC-SHA1',
		timestamp => time,
		nonce => Digest::SHA1::sha1_base64(time . $$ . rand),
		token => $oauth_token,
		verifier => $verifier,
		token_secret => $request_token_secret,
	);
	
	$request->sign;
	return _save_log('Signature verification failed! check OAuth parameters.',$blog_id)
		unless $request->verify;

	my $ua = LWP::UserAgent->new(
		ssl_opts => {
			verify_hostname => 1
		},
	);
	my $http_hdr = HTTP::Headers->new('Authorization' => $request->to_authorization_header);
	my $http_req = HTTP::Request->new($request_method, $access_token_url, $http_hdr, $request->to_post_body);
	my $res = $ua->request($http_req);
	my %param;
	if ($res->is_success) {
		$param{ 'page_title' } = 'Get Access Token Success!';
		my $cfg_plugins_uri = $app->base . $app->uri( mode => 'cfg_plugins', args => { blog_id => $blog_id } );
		$param{ 'msg' } = qq{Get Access Token Success! [<a href="$cfg_plugins_uri" title="Back To Plugin config">Back To Plugin config</a>]};
		$param{ 'is_success' } = 1;
		$param{ 'show_token' } = 1;
		my $response = Net::OAuth->response('access token')->from_post_body($res->content);
		my $access_token = $response->token;
		my $access_token_secret = $response->token_secret;
		my $user_id = $response->extra_params->{'user_id'};
		my $screen_name = $response->extra_params->{'screen_name'};
		
		$plugin->set_config_value( 'access_token', $access_token, $scope );
		$plugin->set_config_value( 'access_secret', $access_token_secret, $scope );
		$plugin->set_config_value( 'rt_twitter_name', $screen_name, $scope );
		
		$param{ 'access_token' } = $access_token;
		$param{ 'access_secret' } = $access_token_secret;
		$param{ 'user_id' } = $user_id;
		$param{ 'screen_name' } = $screen_name;
	}else {
		
		$param{ 'page_title' } = 'Get Access Token failed!';
		my $error_message = $res->as_string;
        $param{ 'msg' } = 'Get Access Token failed!<br />' . $error_message;
        _save_log("Get Access Token failed.$error_message",$blog_id);
	}
	return %param;

}

1;
