#name => PostTwiOauth
#version => 0.50,
#description => 'Post entry title and bit.ly permalink to Twitter with OAuth',
#author_name => 'macminiosx',
#author_link => 'http://www.macminiosx.com/',
#doc_link => 'http://www.macminiosx.com/tag/PostTwiOauth',

package PostTwiOauth::PostTwiOauthReq;

use strict;
use MT;
use MT::App;
use MT::Plugin;
use HTTP::Request::Common;
use LWP::UserAgent;
use Digest::SHA1;
use Net::OAuth;
$Net::OAuth::PROTOCOL_VERSION = Net::OAuth::PROTOCOL_VERSION_1_0A;

our $PLUGIN_NAME = 'PostTwiOauth';

sub _save_log {
	my ($error_mess,$blog_id,$author_id,$remote_ip) = @_;
	use MT::Log; 
	my $log = MT::Log->new;
	$error_mess = $PLUGIN_NAME . ':' . $error_mess;
	$log->blog_id($blog_id);
	$log->author_id($author_id);
	$log->ip($remote_ip) if $remote_ip;
	$log->message($error_mess);
	$log->level(MT::Log::ERROR);
	$log->save
		or die $log->errstr;
	return;
}
sub OauthReq {
	my $app = shift;
	my $blog_id = $app->{'query'}->param('blog_id');
	my $author_id = $app->user->id;
	my $remote_ip = $app->remote_ip;
	return _save_log('Illegal access!',$blog_id, $author_id, $remote_ip) unless $app->validate_magic;
	my $plugin = new MT::Plugin::PostTwiOauth({
    	name => $PLUGIN_NAME,
	});
	my %plugin_params;
	$plugin->load_config(\%plugin_params, 'blog:' . $blog_id);
	my $consumer_key = $plugin_params{'consumer_key'};
	return q() unless $consumer_key;
	my $consumer_secret = $plugin_params{'consumer_secret'};
	return q() unless $consumer_secret;
	my $callback_url = $plugin_params{'callback_url'};
	return q() unless $callback_url;
    
	my $request_token_url = 'https://api.twitter.com/oauth/request_token';
	my $request_method = 'GET';

	my $request = Net::OAuth->request("request token")->new(
		consumer_key => $consumer_key,
		consumer_secret => $consumer_secret,
		request_url => $request_token_url,
		request_method => $request_method,
		signature_method => 'HMAC-SHA1',
		timestamp => time,
		nonce => Digest::SHA1::sha1_base64(time . $$ . rand),
		callback => $callback_url,
	);

	$request->sign;
	return _save_log('Signature verification failed! check OAuth parameters.',$blog_id, $author_id)
		unless $request->verify;

	my $ua = LWP::UserAgent->new(
		ssl_opts => {
			verify_hostname => 1
		},
	);
	my $http_hdr = HTTP::Headers->new('Authorization' => $request->to_authorization_header);
	my $http_req = HTTP::Request->new($request_method, $request_token_url, $http_hdr);
	my $res = $ua->request($http_req);
	
	if ($res->is_success) {
		my $response = Net::OAuth->response('request token')->from_post_body($res->content);
		my $request_token = $response->token;
		my $request_token_secret = $response->token_secret;
		
		my $authorize_url = 'https://api.twitter.com/oauth/authorize?oauth_token=' . $request_token;
		my $cookie = $app->bake_cookie ( -name=>'post2twi',
			-value => {
				setBlog_id => $blog_id,
				token => $request_token,
				token_secret => $request_token_secret,
			},-path=>'/',
		);
		return $app->redirect($authorize_url,UseMeta => 1, -cookie => $cookie);
	}else {
		my $error_message = $res->as_string;
		_save_log("Failed Twitter Authorization.$error_message",$blog_id, $author_id);
		return qq(Failed Twitter Authorization.<br />$error_message);
	}

}

1;
